#include "GetObjects.js"

var objects;
var	last_created = null;

function Update()
{
	var radius = moi.ui.commandUI.radius.value;
	if ( !radius || radius < 0.0001 )
		return;

	var gd = moi.geometryDatabase;
	if ( last_created )
	{
		gd.removeObjects( last_created );
		last_created = null;
	}

	var curves = objects.getCurves();
	for ( var iCrv = 0; iCrv < curves.length; ++iCrv )
	{
		var crv = curves.item(iCrv);
		if ( !crv.isPlanar )
			continue;

		var frame = crv.planarFrame;
		var segs = crv.getSubObjects();
		if ( segs.length == 1 )
			continue;

		var circles = gd.createObjectList();

		for ( var iSeg = 0; iSeg < segs.length; ++iSeg )
		{
			var seg = segs.item(iSeg);
			var next_seg;

			if ( iSeg == segs.length-1 )
			{
				if ( !crv.isClosed )
					break;

				next_seg = segs.item(0);
			}
			else
			{
				next_seg = segs.item(iSeg+1);
			}


			var this_tan = seg.evaluateTangent( seg.domainMax );
			this_tan.scale( -1.0 );
			var next_tan = next_seg.evaluateTangent( next_seg.domainMin );
			var pt = seg.getEndPt();


			var dir = moi.vectorMath.add( this_tan, next_tan );
			var mag = Math.sqrt( dir.x * dir.x + dir.y * dir.y + dir.z * dir.z );
			if ( mag < 0.0001 )
				continue;

			dir.scale( radius / mag );

			frame.origin = moi.vectorMath.add( pt, dir );

			var circle_factory = moi.command.createFactory( 'circle' );
			circle_factory.setInput( 0, true ); // is radius
			circle_factory.setInput( 1, frame ); // frame for center point and x/y axes
			circle_factory.setInput( 3, radius );

			var circle_list = circle_factory.calculate();
			if ( circle_list.length == 1 )
				circles.addObject( circle_list.item(0) );
		}

		var created_objs;

		if ( crv.isClosed )
		{
			var bool_factory;

			if ( moi.ui.commandUI.flip.value )
			{
				bool_factory = moi.command.createFactory( 'booleandifference' );

				var crvlist = gd.createObjectList();
				crvlist.addObject( crv );
				bool_factory.setInput( 0, crvlist );
				bool_factory.setInput( 1, circles );
			}
			else
			{
				bool_factory = moi.command.createFactory( 'booleanunion' );
				circles.addObject( crv );
				bool_factory.setInput( 0, circles );
			}

			bool_factory.update();
			created_objs = bool_factory.getCreatedObjects();
			bool_factory.commit();
		}
		else
		{
			gd.addObjects( circles );
			created_objs = circles;
		}

		if ( last_created == null )
			last_created = gd.createObjectList();

		for ( var iObj = 0; iObj < created_objs.length; ++iObj )
			last_created.addObject( created_objs.item(iObj) );
	}
}

function DoInverseFillet()
{
	var objectpicker = moi.ui.createObjectPicker();
	objectpicker.allowCurves();
	
	if ( !GetObjects( objectpicker ) )
		return;
		
	objects = objectpicker.objects;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectObjectsPrompt' );
	moi.ui.showUI( 'RadiusOptions' );
	moi.ui.endUIUpdate();

	Update();

	var commandDialog = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !commandDialog.waitForEvent() )
			return false;
			
		if ( commandDialog.event == 'done' )
			break;

		if ( commandDialog.event == 'radius' || commandDialog.event == 'flip' )
			Update();
	}
}

DoInverseFillet();
